# README: litmus-tests Project

This project comprises a suite of litmus tests implemented in C, specifically crafted to validate the semantics of Remote Direct Memory Access (RDMA) models. It encompasses both client and server logic tailored to accommodate a variety of litmus test scenarios.

## Dependencies
Make sure the required dependencies (`librdmacm` and `libibverbs`) are present on your system. If not, install them using the following command:



```bash
apt-get install -y build-essential libibverbs-dev librdmacm-dev pkg-config meson
```

- **librdmacm:** This library provides the RDMA Communication Manager (CM) API, which is used for managing RDMA connections.
- **libibverbs:** This library provides the InfiniBand Verbs API, which is used for low-level control of InfiniBand and iWARP devices.


## Building the Project
To build the project, you can use Meson. Simply execute the following commands:

```bash
meson build
ninja -C build
```



## Project Structure
- The `litmus-tests` directory contains C source files that encode both client and server logic. 
- Each litmus test comprises two or more files: one for the client logic and one or more for the server(s), all sharing the same basename. This basename corresponds to the test name mentioned in the paper.




## Running Tests

After building the project, navigate to the `build` directory, where you'll find the executables for both client and server logic.

### Running tests locally

The `litmus-tests/local.sh` script can run the server and the client of a given test on the same host. It receives the name of the test and optionally the IP address of the local RDMA NIC. For example:

```bash
../litmus-tests/local.sh -t mp3bis -i 192.168.0.1
```

The IP address can also be set using the `LOCAL_IP` environment variable.

One can run all the local tests automatically with the `meson test` command.

### Manual tests

To run a litmus test, execute the corresponding executable. It's essential to run both client and server components concurrently. Start the server first, which waits for connections from the client. For example, to run the `LB3` litmus test:

```bash
./lb3_server
./lb3_client <IP_OF_SERVER>
```
Note that certain tests may involve the operation of two servers simultaneously.

### Automated tests

For automation, each litmus test is accompanied by a bash script with the same basename (e.g., `lb3.sh`). When invoked, this script triggers the execution of `remote.sh`, which retrieves network configuration parameters from `config.sh` and orchestrates test scenarios, managing both server and client logic. This script should be executed on the client side.

The configuration file contains the following settings:

| Setting               | Description                                                         |
| --------------------- | ------------------------------------------------------------------- |
| `USER`                | User name for logging to the remote servers through ssh.            |
| `PASSWORD`            | Password for ssh (using sshpass). Leave empty to rely on ssh keys.  |
| `SERVER_IDS`          | An array of server hostnames reachable via ssh. Two entries are needed to run all tests, though the same server can be repeated. |
| `SERVER_IPS`          | An array of server IP addresses, assigned to the servers RDMA NICs. |
| `SERVER_PROGRAM_PATH` | The path to the litmus tests binaries on the server.                |
| `NUM_ITERATIONS`      | Number of times to repeat the test.                                 |

### Background traffic

To generate background traffic and stress the system further, we use the perftest tool suite of benchmark tools. It can be installed with

```bash
apt-get install perftest
```

For example, to generate a stream of RDMA write operations with 512 QPs from the server to the client, we can run on the client:

```bash
ib_write_bw --run_infinitely --report_gbits -F --qp 512
```

and on the server:

```bash
ib_write_bw --run_infinitely --report_gbits -F --qp 512 <client IP address>
```
