/* 
 *  █▀▀ █░░ █ █▀▀ █▄░█ ▀█▀ ▄▄ █▀ █ █▀▄ █▀▀   █░░ █▀█ █▀▀ █ █▀▀
 *  █▄▄ █▄▄ █ ██▄ █░▀█ ░█░ ░░ ▄█ █ █▄▀ ██▄   █▄▄ █▄█ █▄█ █ █▄▄
 *
 *
 * build：
 *   make 2p2w2_client
 *** gcc -g -o 2p2w2_client 2p2w2_client.c rdma_setup_client.c handle_work_completion.c disconnect_cleanup.c -lrdmacm -libverbs
 *
 * usage：
 * ./2p2w2_client 192.168.0.2 <index> // to sgx02
 * 
 *     P1         │   P2
 *     init x=0   │  init y=0
 *     ───────────┤─────────────
 *     x  := 2    │  y  := 2
 *     y² := 1    │  x¹ := 1
 *     ─────────────────────────
 *     (x,y) = (2,2) not possible
 *
 *    ╔════════════════╗              ╔════════════════╗
 *    ║ Client [local) ║              ║ Server (remote)║
 *    ╚════════════════╝              ╚════════════════╝
 *    │ x ... buf[0]   │─── x²:=2 ───>│ y ... buf[0]   │
 *    │                │<── y²:=2 ────│                │
 *    └────────────────┘              └────────────────┘ 
 */
#include "common.h"
#include "rdma_common_client.h"
#include "handle_work_completion.h"
#include "disconnect_cleanup_client.h"
#include "rdma_setup_client.h"
#include "read.h"
#include "write.h"

int main(int argc, char *argv[]) {

	char *itr = (argc > 2) ? argv[2] : "(null)";	
    // Seed the random number generator
    srand(time(0));
	struct client_common client = {};

	bool fence = false;


    	
/*
 * Setup RDMA resources
 * Exchange information about the local buffer between the client and server 
 */
    rdma_setup_client(&client, argv[1], default_config(2 /* variables */, itr));
   
    const local_var  x    = {0},
    				 sync = {1};
    const remote_var y = {0};

    (void)sync;

// Begin of Litmus test

	/* x := 2 */
    store_local(&client.common, x, 2);
    
	/* y^2 := 1 */
    write_inline(&client.common, y, 1, fence);


	/* Wait for the server to indicate the completion of the operation  */
	int ret = handle_work_completion_status_events(client.common.send_cq, 1);
	if (ret != 1) {
		rdma_error("Failed to get 1 work completion from server, ret = %d \n", ret);
		return ret;
	}
    
// End of Litmus test

    /* Wait for the sync message */
    int ret2 = handle_work_completion_status_events(client.common.recv_cq, 1);
    assert(ret2 == 1);


    /* Wait for server to complete before disconnecting. */
    // while (!*y) ;


    // Call the disconnect() function
    int result = disconnect(&client);
    if (result != 0) {
        fprintf(stderr, "Error: disconnect failed with error code %d\n", result);
    }

	printf("Upon Litmus Test execution: x == %u\n",
        load_local(&client.common, x));

    // Call the cleanup function
    result = cleanup(&client);
    if (result != 0) {
        fprintf(stderr, "Error: cleanup failed with error code %d\n", result);
    }    

    return 0;
}



	
	

