/* 
 *  █▀ █▀▀ █▀█ █░█ █▀▀ █▀█ ▄▄ █▀ █ █▀▄ █▀▀   █░░ █▀█ █▀▀ █ █▀▀
 *  ▄█ ██▄ █▀▄ ▀▄▀ ██▄ █▀▄ ░░ ▄█ █ █▄▀ ██▄   █▄▄ █▄█ █▄█ █ █▄▄
 *
 *
 * build:
 * gcc -o crma1_server crma1_server.c -lrdmacm -libverbs
 * 
 * usage: 
 * ./crma1_server <1 or 2> <index>
 * 
 *      P1       │ P2       │ P3
 *      init x=0 │ init y=0 │ init z=0
 *     ──────────┤──────────┤─────────────
 *     x := y²   │ y  := 2  │  z := 1
 *     flush(2)  │          |  b := z
 *     z³ := x   │          │
 *     a := x    │          │
 *     ──────────┤──────────┤──────────
 *     (a,b)=(2,0) not possible
 *
 *    ╔════════════════╗              ╔═════════════════╗
 *    ║ Client (local) ║              ║ Server1 (remote)║
 *    ╚════════════════╝              ╚═════════════════╝
 *    │ x ...    buf[0]│<── x:=y² ────│ y ... buf[0]    │
 *    │                │              └─────────────────┘
 *    │                │              ╔═════════════════╗
 *    │                │              ║ Server2 (remote)║
 *    │                │              ╚═════════════════╝
 *    │                │ ── z³:=x ────│ z ... buf[0]    │
 *    └────────────────┘              └─────────────────┘
 */
#include "common.h"
#include "rdma_common_server.h"
#include "handle_work_completion.h"
#include "disconnect_cleanup_server.h"
#include "rdma_setup_server.h"
#include "read.h"
#include "write.h"

void server1(struct server_common *server) {
    // Define pointer aliases
    const local_var y = {0};
    const remote_var sync1 = {1};

    // /* Wait for the sync message */
	// ret = handle_work_completion_status_events(server1.common.recv_cq, 1);
    // assert(ret == 1);

/*
 ◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤
 ◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣
	
  _     _ _                         _____         _   
 | |   (_) |_ _ __ ___  _   _ ___  |_   _|__  ___| |_ 
 | |   | | __| '_ ` _ \| | | / __|   | |/ _ \/ __| __|
 | |___| | |_| | | | | | |_| \__ \   | |  __/\__ \ |_ 
 |_____|_|\__|_| |_| |_|\__,_|___/   |_|\___||___/\__|

*/
    random_cpu_delay();

	/* y := 2  */
    store_local(&server->common, y, 2);

    write_inline_w_imm(&server->common, sync1, -1, 0x80000000ul, true /* fence */);
}

void server2(struct server_common *server) {
    // Define pointer aliases
    const local_var z = {0};
    const remote_var sync2 = {2};

    // /* Wait for the sync message */
	// ret = handle_work_completion_status_events(server1.common.recv_cq, 1);
    // assert(ret == 1);

/*
 ◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤
 ◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣
	
  _     _ _                         _____         _   
 | |   (_) |_ _ __ ___  _   _ ___  |_   _|__  ___| |_ 
 | |   | | __| '_ ` _ \| | | / __|   | |/ _ \/ __| __|
 | |___| | |_| | | | | | |_| \__ \   | |  __/\__ \ |_ 
 |_____|_|\__|_| |_| |_|\__,_|___/   |_|\___||___/\__|

*/
    random_cpu_delay();

	/* z := 1  */
    store_local(&server->common, z, 1);

    random_cpu_delay();

    /* b := z */
    uint32_t b = load_local(&server->common, z);

    write_inline_w_imm(&server->common, sync2, b, 0x80000000ul, true /* fence */);
}

int main(int argc, char *argv[]) 
{
    if (argc < 2) {
        fprintf(stderr, "Usage:./crma1_server <1/2> [optional iteration]\n");
        return 1;
    }
    int server_node = atoi(argv[1]);
    assert(server_node == 1 || server_node == 2);

	char *iter = (argc > 2) ? argv[2] : "(null)";
	
    // Seed the random number generator
    srand(time(0));

	struct server_common server = {};

/*
 - Setup RDMA resources
 - Exchange information about the local buffer between the client and server 
 */
    struct common_setup_config config = default_config(2 /* variables */, iter);
    if (server_node == 2) {
        config.port++;
    }
	rdma_setup_server(&server, config);

    if (server_node == 1)
        server1(&server);
    else
        server2(&server);
    
/*
 ◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤
 ◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣
*/ 

    // Call the disconnect() function
    int result = disconnect(&server);
    if (result != 0) {
        fprintf(stderr, "Error: disconnect1 failed with error code %d\n", result);
    }

    // Call the cleanup function
    result = cleanup(&server);
    if (result != 0) {
        fprintf(stderr, "Error: cleanup failed with error code %d\n", result);
    }

    return 0;
}
