/* 
 *  █▀▀ █░░ █ █▀▀ █▄░█ ▀█▀ ▄▄ █▀ █ █▀▄ █▀▀   █░░ █▀█ █▀▀ █ █▀▀
 *  █▄▄ █▄▄ █ ██▄ █░▀█ ░█░ ░░ ▄█ █ █▄▀ ██▄   █▄▄ █▄█ █▄█ █ █▄▄
 *
 *
 * build：
 *   make fig2a_client
 *** gcc -g -o fig2a_client fig2a_client.c rdma_setup_client.c handle_work_completion.c disconnect_cleanup.c -lrdmacm -libverbs
 *
 * usage：
 * ./fig2a_client 192.168.0.2 <index> // to sgx02
 * 
 *
 ◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤
 * The server-side metadata (buffer/rkey) is transmitted to the client upon accepting 
 * the connection (rdma_accept), using the rep_pdata mechanism.
 *
 * On the client side, the buffer information metadata is communicated to the server 
 * through a send/recv exchange.
 ◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣
 *
 *     P1         │   P2
 *     init x=0   │  init z=0
 *     ───────────┤─────────────
 *     x  := 1    │  
 *     z² := x    │  
 *     ─────────────────────────
 *            z \in {1}  
 *
 *
 *
 *    ╔════════════════╗              ╔════════════════╗
 *    ║ Client [local) ║              ║ Server (remote)║
 *    ╚════════════════╝              ╚════════════════╝
 *    │ x ... buf[0]   │─── z²:=X ───>│ z ... buf[0]   │
 *    └────────────────┘              └────────────────┘ 
 *                   
 *                                    
 */
#include "common.h"
#include "rdma_common_client.h"
#include "handle_work_completion.h"
#include "disconnect_cleanup_client.h"
#include "rdma_setup_client.h"
#include "read.h"
#include "write.h"

int main(int argc, char *argv[]) {

	char *itr = (argc > 2) ? argv[2] : "(null)";	
    // Seed the random number generator
    srand(time(0));
	struct client_common client = {};

	bool fence = false;


    	
/*
 * Setup RDMA resources
 * Exchange information about the local buffer between the client and server 
 */
    rdma_setup_client(&client, argv[1], default_config(1 /* variables */, itr));
   
    const local_var x = {0};
    const remote_var z = {0};

// Begin of Litmus test

    store_local(&client.common, x, 1);
    // printf("\t\tx := %u\n", *x);



	// printf("\t\tAdding delay between z^2 := x and x := 1\n");
	 // random_cpu_delay();

	/* z^2 := x */
    write_to_from(&client.common, z, x, fence);

	/* Wait for the server to indicate the completion of the operation  */
	int ret = handle_work_completion_status_events(client.common.send_cq, 1);
	if(ret != 1) {
		rdma_error("Failed to get 1 work completion from server, ret = %d \n", ret);
		return ret;
	}


    
// End of Litmus test




    // Call the disconnect() function
    int result = disconnect(&client);
    if (result != 0) {
        fprintf(stderr, "Error: disconnect failed with error code %d\n", result);
    }




    // Call the cleanup function
    result = cleanup(&client);
    if (result != 0) {
        fprintf(stderr, "Error: cleanup failed with error code %d\n", result);
    }    

    return 0;
}



	
	

