/* 
 *  █▀▀ █░░ █ █▀▀ █▄░█ ▀█▀ ▄▄ █▀ █ █▀▄ █▀▀   █░░ █▀█ █▀▀ █ █▀▀
 *  █▄▄ █▄▄ █ ██▄ █░▀█ ░█░ ░░ ▄█ █ █▄▀ ██▄   █▄▄ █▄█ █▄█ █ █▄▄
 *
 *
 * build：
 *   make fig2d_client
 *** gcc -g -o fig2d_client fig2d_client.c rdma_setup_client.c handle_work_completion.c disconnect_cleanup.c -lrdmacm -libverbs
 *
 * usage：
 * ./fig2d_client 192.168.0.2 <index> // to sgx02
 * 
 *
 ◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤
 * The server-side metadata (buffer/rkey) is transmitted to the client upon accepting 
 * the connection (rdma_accept), using the rep_pdata mechanism.
 *
 * On the client side, the buffer information metadata is communicated to the server 
 * through a send/recv exchange.
 ◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣
 *
 *     P1         │   P2
 *     init x = 0 │  init z = 0
 *     ───────────┤─────────────
 *     z² := x    │  
 *     z² := x    │  
 *     poll(2)    │  
 *     x  := 1    │  
 *     ─────────────────────────
 *            z \in {0,1}  
 *
 *
 *
 *    ╔════════════════╗              ╔════════════════╗
 *    ║ Client [local) ║              ║ Server (remote)║
 *    ╚════════════════╝              ╚════════════════╝
 *    │ x ... buf[0]   │─── z²:=x ───>│ z ... buf[0]   │
 *    └────────────────┘              └────────────────┘ 
 *                   
 *                                    
 */
#include "common.h"
#include "rdma_common_client.h"
#include "handle_work_completion.h"
#include "disconnect_cleanup_client.h"
#include "rdma_setup_client.h"
#include "read.h"
#include "write.h"

int main(int argc, char *argv[]) {

	char *itr = (argc > 2) ? argv[2] : "(null)";	
    // Seed the random number generator
    srand(time(0));
	struct client_common client = {};
	struct ibv_wc wc[2];

	bool fence = false;
	
/*
 * Setup RDMA resources
 * Exchange information about the local buffer between the client and server 
 */
    rdma_setup_client(&client, argv[1], default_config(1 /* variables */, itr));
   






// Begin of Litmus test

    const local_var x = {0};
	const remote_var z = {0};
    
	random_nic_delay(&client.common);

	/* z^2 := x */
    write_to_from(&client.common, z, x, fence);

	/* z^2 := x */
    write_to_from(&client.common, z, x, fence);

	int first_ret = handle_work_completion_status_events_wc(client.common.send_cq, 1, 2, wc);
	if(first_ret < 1) {
		rdma_error("Failed to get 1 work completion from server, ret = %d \n", first_ret);
		return first_ret;
	}

    store_local(&client.common, x, 1);

	int ret = handle_work_completion_status_events(client.common.send_cq, 2 - first_ret);
	if(ret != 2 - first_ret) {
		rdma_error("Failed to get 1 work completion from server, ret = %d \n", ret);
		return ret;
	}

	printf("Got %d completions on first poll\n", first_ret);
    
// End of Litmus test




    // Call the disconnect() function
    int result = disconnect(&client);
    if (result != 0) {
        fprintf(stderr, "Error: disconnect failed with error code %d\n", result);
    }




    // Call the cleanup function
    result = cleanup(&client);
    if (result != 0) {
        fprintf(stderr, "Error: cleanup failed with error code %d\n", result);
    }    

    return 0;
}



	
	

