/* 
 *  █▀ █▀▀ █▀█ █░█ █▀▀ █▀█ ▄▄ █▀ █ █▀▄ █▀▀   █░░ █▀█ █▀▀ █ █▀▀
 *  ▄█ ██▄ █▀▄ ▀▄▀ ██▄ █▀▄ ░░ ▄█ █ █▄▀ ██▄   █▄▄ █▄█ █▄█ █ █▄▄
 *
 *
 * build:
 * gcc -o fig2e_server fig2e_server.c -lrdmacm -libverbs
 * 
 * usage: 
 * ./fig2e_server <1 or 2> <index>
 * 
 *     P1         │   P2        │   P3
 *     init x=0   │  init z=1   │  init y=2 
 *     ───────────┤─────────────┤───────────
 *     x  := z²   │             │
 *     x  := y³   │             │
 *     poll(2)    │             │
 *     poll(3)    │             │
 *     ──────────────────────────────────────
 *       x == {1,2} permitted 
 *
 *
 *
 *    ╔════════════════╗              ╔═════════════════╗              ╔═════════════════╗
 *    ║ Client [local) ║              ║ Server1 (remote)║              ║ Server2 (remote)║
 *    ╚════════════════╝              ╚═════════════════╝              ╚═════════════════╝
 *    │ x ... buf[0]   │<── x:=z² ────│ z ... buf[0]    │  ┌ x:= y³ ───│ y ... buf[0]    │
 *    └────────────────┘              └─────────────────┘  │           └─────────────────┘ 
 *            └──<─────────────────────────────────────────┘
 *
 * 
 * 
 */
#include "common.h"
#include "rdma_common_server.h"
#include "handle_work_completion.h"
#include "disconnect_cleanup_server.h"
#include "rdma_setup_server.h"
#include "read.h"
#include "write.h"

int main(int argc, char *argv[]) 
{
    if (argc < 2) {
        fprintf(stderr, "Usage:./fig2e_server <1/2> [optional iteration]\n");
        return 1;
    }
    int server_node = atoi(argv[1]);
    assert(server_node == 1 || server_node == 2);

	char *iter = (argc > 2) ? argv[2] : "(null)";

    // Seed the random number generator
    srand(time(0));

	struct server_common server = {};

    /*
    - Setup RDMA resources
    - Exchange information about the local buffer between the client and server 
    */
    struct common_setup_config config = default_config(1 /* variables */, iter);
    if (server_node == 2) {
        config.port++;
    }
    config.variables_initialization_value = server_node;
    rdma_setup_server(&server, config);

/*
 ◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤
 ◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣
*/ 

    // Call the disconnect() function
    int result = disconnect(&server);
    if (result != 0) {
        fprintf(stderr, "Error: disconnect1 failed with error code %d\n", result);
    }

    // Call the cleanup function
    result = cleanup(&server);
    if (result != 0) {
        fprintf(stderr, "Error: cleanup failed with error code %d\n", result);
    }

    return 0;
}
