/* 
 *  █▀ █▀▀ █▀█ █░█ █▀▀ █▀█ ▄▄ █▀ █ █▀▄ █▀▀   █░░ █▀█ █▀▀ █ █▀▀
 *  ▄█ ██▄ █▀▄ ▀▄▀ ██▄ █▀▄ ░░ ▄█ █ █▄▀ ██▄   █▄▄ █▄█ █▄█ █ █▄▄
 *
 *
 * build:
 * make fig2f_server
 * 
 * usage: 
 * ./fig2f_server <1 or 2> <index> 
 * 
 *
 ◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤
 * The server-side metadata (buffer/rkey) is transmitted to the client upon accepting 
 * the connection (rdma_accept), using the rep_pdata mechanism.
 *
 * On the client side, the buffer information metadata is communicated to the server 
 * through a send/recv exchange.
 ◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣
 *
 *     P1         │   P2        │   P3
 *     init x=0   │  init z=1   │  init y=2 
 *     ───────────┤─────────────┤───────────
 *     y³ := x    │             │
 *     x  := z²   │             │
 *     poll(2)    │             │
 *     poll(3)    │             │
 *     ──────────────────────────────────────
 *       y == {0,1} permitted 
 *
 *
 *
 *    ╔════════════════╗              ╔═════════════════╗              ╔═════════════════╗
 *    ║ Client [local) ║              ║ Server1 (remote)║              ║ Server2 (remote)║
 *    ╚════════════════╝              ╚═════════════════╝              ╚═════════════════╝
 *    │ x ... buf[0]   │<── x:=z² ────│ z ... buf[0]    │  ┌ y³:= x ──>│ y ... buf[0]    │
 *    └───────┬────────┘              └─────────────────┘  │           └─────────────────┘ 
 *            └────────────────────────────────────────────┘
 *
 * 
 */
#include "common.h"
#include "rdma_common_server.h"
#include "handle_work_completion.h"
#include "disconnect_cleanup_server.h"
#include "rdma_setup_server.h"
#include "read.h"
#include "write.h"

int main(int argc, char *argv[]) 
{ 
    if (argc < 2) {
        fprintf(stderr, "Usage:./fig2e_server <1/2> [optional iteration]\n");
        return 1;
    }
    int server_node = atoi(argv[1]);
    assert(server_node == 1 || server_node == 2);

	char *iter = (argc > 2) ? argv[2] : "(null)";

    // Seed the random number generator
    srand(time(0));
    
	struct server_common server = {};

/*
  - Setup RDMA resources
 - Exchange information about the local buffer between the client and server 
 */
struct common_setup_config config = default_config(1 /* variables */, iter);
    if (server_node == 2) {
        config.port++;
    }
	config.variables_initialization_value = server_node;
    rdma_setup_server(&server, config);

	const local_var server2_y = {0};
    		    	 	
/*
 ◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤
 ◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣
*/ 

// Call the disconnect() function
    int result = disconnect(&server);
    if (result != 0) {
        fprintf(stderr, "Error: disconnect1 failed with error code %d\n", result);
    }

	if (server_node == 2) {
    	printf("[%s] Upon Litmus Test execution: y == %u\n",
        	iter, load_local(&server.common, server2_y));
	}

    // Call the cleanup function
    result = cleanup(&server);
    if (result != 0) {
        fprintf(stderr, "Error: cleanup failed with error code %d\n", result);
    }

    return 0;
}
