/* 
 *  █▀▀ █░░ █ █▀▀ █▄░█ ▀█▀ ▄▄ █▀ █ █▀▄ █▀▀   █░░ █▀█ █▀▀ █ █▀▀
 *  █▄▄ █▄▄ █ ██▄ █░▀█ ░█░ ░░ ▄█ █ █▄▀ ██▄   █▄▄ █▄█ █▄█ █ █▄▄
 *
 *
 * build：
 *   make fig2h_client
 *** gcc -g -o fig2h_client fig2h_client.c rdma_setup_client.c handle_work_completion.c disconnect_cleanup.c -lrdmacm -libverbs
 *
 * usage：
 * ./fig2h_client 192.168.0.2 <index> // to sgx02
 * 
 *
 ◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤
 * The server-side metadata (buffer/rkey) is transmitted to the client upon accepting 
 * the connection (rdma_accept), using the rep_pdata mechanism.
 *
 * On the client side, the buffer information metadata is communicated to the server 
 * through a send/recv exchange.
 ◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣
 *
 *     P1         │   P2
 *     init x = 1 │  init y=z=0
 *     ───────────┤─────────────
 *     x  := y²   │  
 *     z² := x    │  
 *     poll(2)    │ 
 *     ─────────────────────────
 *            z == {0,1}  
 *
 *
 *
 *    ╔════════════════╗              ╔════════════════╗
 *    ║ Client [local) ║              ║ Server (remote)║
 *    ╚════════════════╝              ╚════════════════╝
 *    │ x ... buf[0]   │<── x:=y² ────│ y ... buf[0]   │
 *    └────────────────┘              ├────────────────┤ 
 *                   └───── z²:=x ───>│ z ... buf[1]   │
 *                                    └────────────────┘
 */
#include "common.h"
#include "rdma_common_client.h"
#include "handle_work_completion.h"
#include "disconnect_cleanup_client.h"
#include "rdma_setup_client.h"
#include "read.h"
#include "write.h"

int main(int argc, char *argv[]) {

	char *itr = (argc > 2) ? argv[2] : "(null)";	
    // Seed the random number generator
    srand(time(0));
	struct client_common client = {};


/*
 * Setup RDMA resources
 * Exchange information about the local buffer between the client and server 
 */
    rdma_setup_client(&client, argv[1], default_config(1 /* variables */, itr));
   






// Begin of Litmus test

    const local_var x = {0};
    const remote_var y = {0},
                     z = {1};
    
    store_local(&client.common, x, 1);

    /* x := y^2 */
    read_to_from(&client.common, x, y);

	// No printing within the test, as they can increase the delay:
	// printf("\t\tAdding delay between x := y^2 and z^2 := x\n");
	random_cpu_delay();

	/* z^2 := x */
    write_to_from(&client.common, z, x, false /* fence */);

	/* Wait for the server to indicate the completion of the two operations  */
	int ret = handle_work_completion_status_events(client.common.send_cq, 2);
	if(ret != 2) {
		rdma_error("Failed to get 2 work completion from server, ret = %d \n", ret);
		return ret;
	}


// End of Litmus test




    // Call the disconnect() function
    int result = disconnect(&client);
    if (result != 0) {
        fprintf(stderr, "Error: disconnect failed with error code %d\n", result);
    }




    // Call the cleanup function
    result = cleanup(&client);
    if (result != 0) {
        fprintf(stderr, "Error: cleanup failed with error code %d\n", result);
    }    

    return 0;
}



	
	

