#include "common.h"
#include "handle_work_completion.h"

#include <infiniband/verbs.h>

#include <errno.h>

int handle_work_completion_status_events_wc(struct ibv_cq *cq, int block_wc, int max_wc, struct ibv_wc *wc) {
    int ret = -1, i, total_wc = 0;

    if (!max_wc)
        return 0;

    total_wc = 0;
    do {
        ret = ibv_poll_cq(cq, max_wc - total_wc, wc + total_wc);
        if (ret < 0) {
            rdma_error("Failed to poll cq for wc due to %d \n", ret);
            return ret;
        }
        total_wc += ret;
    } while (total_wc < block_wc);

    debug("%d WC are completed \n", total_wc);

    for (i = 0; i < total_wc; i++) {
        if (wc[i].status != IBV_WC_SUCCESS) {
            rdma_error("Work completion (WC) has error status: %s at index %d wr_id %ld\n", ibv_wc_status_str(wc[i].status), i, wc[i].wr_id);
            return -(wc[i].status);
        }
    }

    return total_wc;
}

int handle_work_completion_status_events(struct ibv_cq *cq, int max_wc) {
    struct ibv_wc wc[max_wc];

    return handle_work_completion_status_events_wc(cq, max_wc, max_wc, wc);
}
