/* 
 *  █▀ █▀▀ █▀█ █░█ █▀▀ █▀█ ▄▄ █▀ █ █▀▄ █▀▀   █░░ █▀█ █▀▀ █ █▀▀
 *  ▄█ ██▄ █▀▄ ▀▄▀ ██▄ █▀▄ ░░ ▄█ █ █▄▀ ██▄   █▄▄ █▄█ █▄█ █ █▄▄
 *
 *
 * build:
 * gcc -o lb1_server lb1_server.c -lrdmacm -libverbs
 * 
 * usage: 
 * ./lb1_server <index>
 * 
 *
 ◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤
 * The server-side metadata (buffer/rkey) is transmitted to the client upon accepting 
 * the connection (rdma_accept), using the rep_pdata mechanism.
 *
 * On the client side, the buffer information metadata is communicated to the server 
 * through a send/recv exchange.
 ◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣
 *
 *     P1         │   P2
 *     init y=0   │  init x=0
 *     ───────────┤─────────────
 *     a  := y    │  b  := x 
 *     x² := 1    │  y¹ := 1
 *     ─────────────────────────
 *       (a,b)=(1,1) not permitted
 *
 *
 *    ╔════════════════╗              ╔════════════════╗
 *    ║ Client [local) ║              ║ Server (remote)║
 *    ╚════════════════╝              ╚════════════════╝
 *    │ y ... buf[0]   │    x²:=1 ───>│ x ... buf[0]   │
 *    │                │<── y¹:= 1    │                │
 *    └────────────────┘              └────────────────┘
 */
#include "common.h"
#include "rdma_common_server.h"
#include "handle_work_completion.h"
#include "disconnect_cleanup_server.h"
#include "rdma_setup_server.h"
#include "write.h"
#include "read.h"

#include <assert.h>

int main(int argc, char *argv[]) 
{ 
	char *iter = (argc > 1) ? argv[1] : "(null)";
	
    // Seed the random number generator
    srand(time(0));

	struct server_common server = {};

/*
 - Setup RDMA resources
 - Exchange information about the local buffer between the client and server 
 */
	rdma_setup_server(&server, default_config(1 /* variables */, iter));




		
	
/*
 ◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤
 ◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣
	
  _     _ _                         _____         _   
 | |   (_) |_ _ __ ___  _   _ ___  |_   _|__  ___| |_ 
 | |   | | __| '_ ` _ \| | | / __|   | |/ _ \/ __| __|
 | |___| | |_| | | | | | |_| \__ \   | |  __/\__ \ |_ 
 |_____|_|\__|_| |_| |_|\__,_|___/   |_|\___||___/\__|

*/

    const local_var x = {0};

    const remote_var y    = {0},
    			     sync = {1};

    uint32_t b = load_local(&server.common, x);
    	    
	/* y¹ := 1 */
    write_inline(&server.common, y, 1, false /* fence */);
    
	/* Wait for the client to indicate the completion of the operation  */
	int ret = handle_work_completion_status_events(server.common.send_cq, 1);
	if (ret != 1) {
			rdma_error("Failed to get 1 work completion from client, ret = %d \n", ret);
			return ret;
	}  		    	 	
    

/*
 ◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤
 ◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣
*/ 

    write_inline_w_imm(&server.common, sync, b, 0x80000000ul, true /* fence */);

    // Call the disconnect() function
    int result = disconnect(&server);
    if (result != 0) {
        fprintf(stderr, "Error: disconnect failed with error code %d\n", result);
    }

    printf("Upon Litmus Test execution: b == %u\n", b);

    // Call the cleanup function
    result = cleanup(&server);
    if (result != 0) {
        fprintf(stderr, "Error: cleanup failed with error code %d\n", result);
    }

		
    return 0;
}
