#!/bin/bash -e

# set a default value for the LOCAL_IP if not provided externally
LOCAL_IP=${LOCAL_IP:-"192.168.0.1"}

# Parse command line with getopt
while getopts "i:t:n:" opt; do
    case $opt in
        i)
            LOCAL_IP=$OPTARG
            ;;
        n)
            ITERATION=$OPTARG
            ;;
        t)
            TEST=$OPTARG
            ;;
        \?)
            echo "Invalid option: -$OPTARG" >&2
            exit 1
    esac
done

# Check if a test (TEST) is specified
if [ -z "$TEST" ]; then
    echo "No test specified" >&2
    exit 1
fi

num_servers=1
two_servers_tests=(crma1 fig2e fig2f)
if [[ "${two_servers_tests[@]}"  =~ "${TEST}" ]] ; then
    num_servers=2
fi


# assign the path to the server/client executable
SERVER=./${TEST}_server
CLIENT=./${TEST}_client

# Remove file on exit
SERVER_OUTPUT=()
SERVER_PID=()
CLIENT_ARGS=()
for ((i=0; i<$num_servers; i++)); do
    # Allocate temporary file with mktemp
    SERVER_OUTPUT+=("$(mktemp /tmp/server-output-XXXXX.txt)")
    trap_line="cat ${SERVER_OUTPUT[@]} ; rm -f ${SERVER_OUTPUT[@]}"
    trap "$trap_line" EXIT

    SERVER_ARGS=("$ITERATION")
    if [[ "$num_servers" -gt 1 ]] ; then
        SERVER_ARGS=( "$(($i + 1))" "$ITERATION")
    fi
    
    # start the server process(es) 
    "$SERVER" "${SERVER_ARGS[@]}" > "$SERVER_OUTPUT[$i]" &
    SERVER_PID+=($!)

	# monitor the server output to ensure it's successfully listening before proceeding
    if ! timeout 1 tail -f -n+1 "$SERVER_OUTPUT[$i]" | grep -m1 listening > /dev/null ; then
        kill $SERVER_PID[$@]
        exit 1
    fi

    CLIENT_ARGS+=("$LOCAL_IP")
done

# start the client process 
"$CLIENT" "${CLIENT_ARGS[@]}" "$ITERATION"

# wait for all processes to finish before exiting
wait
