/* 
 *  █▀ █▀▀ █▀█ █░█ █▀▀ █▀█ ▄▄ █▀ █ █▀▄ █▀▀   █░░ █▀█ █▀▀ █ █▀▀
 *  ▄█ ██▄ █▀▄ ▀▄▀ ██▄ █▀▄ ░░ ▄█ █ █▄▀ ██▄   █▄▄ █▄█ █▄█ █ █▄▄
 *
 *
 * build:
 * gcc -o mp2_server mp2_server.c -lrdmacm -libverbs
 * 
 * usage: 
 * ./mp2_server <index>
 * 
 *
 ◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤
 * The server-side metadata (buffer/rkey) is transmitted to the client upon accepting 
 * the connection (rdma_accept), using the rep_pdata mechanism.
 *
 * On the client side, the buffer information metadata is communicated to the server 
 * through a send/recv exchange.
 ◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣
 *
 *     P1         │   P2
 *     init x=0   │  init y=0
 *     ───────────┤─────────────
 *     x := 1     │  a := y
 *     y²:= 1     │  b := x¹ 
 *     ─────────────────────────
 *       (a,b)=(1,0) not possible
 *
 *
 *
 *    ╔════════════════╗              ╔════════════════╗
 *    ║ Client [local) ║─┐            ║ Server (remote)║
 *    ╚════════════════╝ │            ╚════════════════╝
 *    │ x ... buf[0]   │ └─ y²:= 1 ──>│ y ... buf[0]   │
 *    │                │─┐            ├────────────────┤
 *    └────────────────┘ └─ b := x¹ ─>│ b ... buf[1]   │ 
 *                                    └────────────────┘
 */
#include "common.h"
#include "rdma_common_server.h"
#include "handle_work_completion.h"
#include "disconnect_cleanup_server.h"
#include "rdma_setup_server.h"
#include "write.h"
#include "read.h"

#include <assert.h>

int main(int argc, char *argv[]) 
{ 
	char *iter = (argc > 1) ? argv[1] : "(null)";
	
    // Seed the random number generator
    srand(time(0));

	struct server_common server = {};

/*
 - Setup RDMA resources
 - Exchange information about the local buffer between the client and server 
 */
	rdma_setup_server(&server, default_config(2 /* variables */, iter));




		
	
/*
 ◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤
 ◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣
	
  _     _ _                         _____         _   
 | |   (_) |_ _ __ ___  _   _ ___  |_   _|__  ___| |_ 
 | |   | | __| '_ ` _ \| | | / __|   | |/ _ \/ __| __|
 | |___| | |_| | | | | | |_| \__ \   | |  __/\__ \ |_ 
 |_____|_|\__|_| |_| |_|\__,_|___/   |_|\___||___/\__|

*/

    const local_var y = {0},
                    b = {1};

    const remote_var x    = {0},
	                 sync = {1};

    // Local
    uint32_t a = -1;

    while (a != 1) {
    
    	a = load_local(&server.common, y);
    		    
	    /* b := x^1 */
	    read_to_from(&server.common, b, x);



            /* Wait for the client to indicate the completion of the operation  */
            int ret = handle_work_completion_status_events(server.common.send_cq, 1);
            if (ret != 1) {
                    rdma_error("Failed to get 1 work completion from client, ret = %d \n", ret);
                    return ret;
            }  		    	 	
    }
/*
 ◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤
 ◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣
*/ 

    write_inline_w_imm(&server.common, sync, 0, 0x80000000ul, true /* fence */);

    // Call the disconnect() function
    int result = disconnect(&server);
    if (result != 0) {
        fprintf(stderr, "Error: disconnect failed with error code %d\n", result);
    }

    printf("Upon Litmus Test execution: (a,b) == (%u,%u)\n",
        a, load_local(&server.common, b));

    // Call the cleanup function
    result = cleanup(&server);
    if (result != 0) {
        fprintf(stderr, "Error: cleanup failed with error code %d\n", result);
    }

		
    return 0;
}
