/* 
 *  █▀▀ █░░ █ █▀▀ █▄░█ ▀█▀ ▄▄ █▀ █ █▀▄ █▀▀   █░░ █▀█ █▀▀ █ █▀▀
 *  █▄▄ █▄▄ █ ██▄ █░▀█ ░█░ ░░ ▄█ █ █▄▀ ██▄   █▄▄ █▄█ █▄█ █ █▄▄
 *
 *
 * build：
 *   make mp4_client
 *** gcc -g -o mp4_client mp4_client.c rdma_setup_client.c handle_work_completion.c disconnect_cleanup.c -lrdmacm -libverbs
 *
 * usage：
 * ./mp4_client 192.168.0.2 <index> // to sgx02
 *
 *     P1         │   P2
 *     init x,y=0 │
 *     ───────────┤─────────────
 *     x := 1     │  a := y¹
 *     y := 1     │  b := x¹
 *     ─────────────────────────
 *       (a,b)=(1,0) possible
 *
 *
 *
 *    ╔════════════════╗              ╔════════════════╗
 *    ║ Client [local) ║              ║ Server (remote)║
 *    ╚════════════════╝              ╚════════════════╝
 *    │ y ... buf[0]   │─── a:=y¹ ───>│ a ... buf[0]   │
 *    ├────────────────┤              ├────────────────┤
 *    │ x ... buf[1]   │─── b:=x¹ ───>│ b ... buf[1]   │
 *    └────────────────┘              └────────────────┘ 
 */
#include "common.h"
#include "rdma_common_client.h"
#include "handle_work_completion.h"
#include "disconnect_cleanup_client.h"
#include "rdma_setup_client.h"
#include "read.h"
#include "write.h"

#include <assert.h>

int main(int argc, char *argv[]) {

	char *itr = (argc > 2) ? argv[2] : "(null)";	
    // Seed the random number generator
    srand(time(0));
	struct client_common client = {};

/*
 * Setup RDMA resources
 * Exchange information about the local buffer between the client and server 
 */

    struct common_setup_config config = default_config(3 /* variables */, itr);
    config.randomize_variable_home = true;
    rdma_setup_client(&client, argv[1], config);
   
    const local_var y    = {0},
                    x    = {1},
                    sync = {2};

    (void)sync;

    // random_cpu_delay();
    random_nic_delay(&client.common);
    
// Begin of Litmus test
    
    store_local(&client.common, x, 1);

	store_local(&client.common, y, 1);

// End of Litmus test

    /* Wait for the sync message */
    int ret = handle_work_completion_status_events(client.common.recv_cq, 1);
    assert(ret == 1);

    // Call the disconnect() function
    int result = disconnect(&client);
    if (result != 0) {
        fprintf(stderr, "Error: disconnect failed with error code %d\n", result);
    }

    // Call the cleanup function
    result = cleanup(&client);
    if (result != 0) {
        fprintf(stderr, "Error: cleanup failed with error code %d\n", result);
    }    

    return 0;
}



	
	

