/* 
 *  █▀ █▀▀ █▀█ █░█ █▀▀ █▀█ ▄▄ █▀ █ █▀▄ █▀▀   █░░ █▀█ █▀▀ █ █▀▀
 *  ▄█ ██▄ █▀▄ ▀▄▀ ██▄ █▀▄ ░░ ▄█ █ █▄▀ ██▄   █▄▄ █▄█ █▄█ █ █▄▄
 *
 *
 * build:
 * gcc -o rrpr3_server rrpr3_server.c -lrdmacm -libverbs
 * 
 * usage: 
 * ./rrpr3_server <index>
 * 
 *     P1        │   P2
 *     init x,y=1│  init z=0
 *     ──────────┤─────────────
 *     x  := z²  │  
 *     y  := z²  │ 
 *     a  := y   │  
 *     b  := x   │  
 *     ─────────────────────────
 *     (a,b)=(0,1) not possible
 *
 *    ╔════════════════╗              ╔════════════════╗
 *    ║ Client (local) ║              ║ Server (remote)║
 *    ╚════════════════╝              ╚════════════════╝
 *    │ x ... buf[0]   │<── x:=z² ────│                │
 *    ├────────────────┤              │ z ... buf[0]   │ 
 *    │ y ... buf[1]   │<── y:=z² ────│                │
 *    └────────────────┘              └────────────────┘
 */
#include "common.h"
#include "rdma_common_server.h"
#include "handle_work_completion.h"
#include "disconnect_cleanup_server.h"
#include "rdma_setup_server.h"
#include "read.h"
#include "write.h"

int main(int argc, char *argv[]) 
{ 
	char *iter = (argc > 1) ? argv[1] : "(null)";
	
    // Seed the random number generator
    srand(time(0));

	struct server_common server = {};

/*
 - Setup RDMA resources
 - Exchange information about the local buffer between the client and server 
 */
	rdma_setup_server(&server, default_config(2 /* variables */, iter));

    // Define pointer aliases
    const local_var z = {0};
    
    const remote_var x    = {0},
    				 y    = {1};

    (void)x; (void)y; (void)z;

/*
 ◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤
 ◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣
	
  _     _ _                         _____         _   
 | |   (_) |_ _ __ ___  _   _ ___  |_   _|__  ___| |_ 
 | |   | | __| '_ ` _ \| | | / __|   | |/ _ \/ __| __|
 | |___| | |_| | | | | | |_| \__ \   | |  __/\__ \ |_ 
 |_____|_|\__|_| |_| |_|\__,_|___/   |_|\___||___/\__|

*/
    
/*
 ◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤
 ◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣◥◤◢◣
*/ 

    // Call the disconnect() function
    int result = disconnect(&server);
    if (result != 0) {
        fprintf(stderr, "Error: disconnect failed with error code %d\n", result);
    }


    // Call the cleanup function
    result = cleanup(&server);
    if (result != 0) {
        fprintf(stderr, "Error: cleanup failed with error code %d\n", result);
    }

		
    return 0;
}
