/* 
 *  █▀▀ █░░ █ █▀▀ █▄░█ ▀█▀ ▄▄ █▀ █ █▀▄ █▀▀   █░░ █▀█ █▀▀ █ █▀▀
 *  █▄▄ █▄▄ █ ██▄ █░▀█ ░█░ ░░ ▄█ █ █▄▀ ██▄   █▄▄ █▄█ █▄█ █ █▄▄
 *
 *
 * build：
 *   make sb1_client
 *** gcc -g -o sb1_client sb1_client.c rdma_setup_client.c handle_work_completion.c disconnect_cleanup.c -lrdmacm -libverbs
 *
 * usage：
 * ./sb1_client 192.168.0.2 <index> // to sgx02
 * 
 *
 ◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤
 * The server-side metadata (buffer/rkey) is transmitted to the client upon accepting 
 * the connection (rdma_accept), using the rep_pdata mechanism.
 *
 * On the client side, the buffer information metadata is communicated to the server 
 * through a send/recv exchange.
 ◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣◥◣
 *
 *     P1         │   P2
 *     init x=0   │  init y=0
 *     ───────────┤─────────────
 *     x := 1     │  y := 1
 *     a := y²    │  b := x¹ 
 *     ─────────────────────────
 *       (a,b)=(0,0) not possible
 *
 *
 *
 *    ╔════════════════╗              ╔════════════════╗
 *    ║ Client [local) ║              ║ Server (remote)║
 *    ╚════════════════╝              ╚════════════════╝
 *    │ x ... buf[0]   │── b := x¹ ──>│ b ... buf[0]   │
 *    ├────────────────┤              ├────────────────┤
 *    │ a ... buf[1]   │<── b := y² ──│ y ... buf[1]   │ 
 *    └────────────────┘              └────────────────┘
 *
 */
#include "common.h"
#include "rdma_common_client.h"
#include "handle_work_completion.h"
#include "disconnect_cleanup_client.h"
#include "rdma_setup_client.h"
#include "read.h"
#include "write.h"

#include <assert.h>

int main(int argc, char *argv[]) {

	char *itr = (argc > 2) ? argv[2] : "(null)";	
    // Seed the random number generator
    srand(time(0));
	struct client_common client = {};

/*
 * Setup RDMA resources
 * Exchange information about the local buffer between the client and server 
 */
    rdma_setup_client(&client, argv[1], default_config(3 /* variables */, itr));
   
    const local_var x    = {0},
    		        a    = {1},
                    sync = {2};
    const remote_var b = {0},
                     y = {1};

    (void)sync;
    (void)b;

    // random_cpu_delay();
    random_nic_delay(&client.common);
    
// Begin of Litmus test
    
    store_local(&client.common, x, 1);
    	
	/* a := y² */
    read_to_from(&client.common, a, y);



	/* Wait for the server to indicate the completion of the operation  */
	int ret = handle_work_completion_status_events(client.common.send_cq, 1);
	if (ret != 1) {
		rdma_error("Failed to get 1 work completion from server, ret = %d \n", ret);
		return ret;
	}
    
// End of Litmus test

    /* Wait for the sync message */
    ret = handle_work_completion_status_events(client.common.recv_cq, 1);
    assert(ret == 1);

    // Call the disconnect() function
    int result = disconnect(&client);
    if (result != 0) {
        fprintf(stderr, "Error: disconnect failed with error code %d\n", result);
    }


	printf("Upon Litmus Test execution: a == %u\n",
        load_local(&client.common, a));


    // Call the cleanup function
    result = cleanup(&client);
    if (result != 0) {
        fprintf(stderr, "Error: cleanup failed with error code %d\n", result);
    }    

    return 0;
}



	
	

