#include "common.h"
#include "rdma_common_client.h"
#include "handle_work_completion.h"
#include "write.h"

#include <assert.h>

void write_to_from(struct common *common, remote_var remote, local_var local, bool fence) {
    struct ibv_sge sge;
    struct ibv_send_wr send_wr = {};
    struct ibv_send_wr *bad_send_wr;

	/* Fill up SGE     */
	set_local_var(common, &sge, local);
	
	/* Link to the send work request */
	send_wr.wr_id                 = 0;
	send_wr.opcode                = IBV_WR_RDMA_WRITE;  
	
	/* Set IBV_SEND_SIGNALED flag will cause an ibv event recv at sender when data transmit from memory to NIC */
    if (fence) {
        send_wr.send_flags        = IBV_SEND_SIGNALED | IBV_SEND_FENCE;
    } else {
        send_wr.send_flags        = IBV_SEND_SIGNALED;
    }

	send_wr.sg_list               = &sge;
	send_wr.num_sge               = 1;
	
	set_remote_var(common, &send_wr, remote);

	/* Post it */
	if (ibv_post_send(common->cm_id->qp, &send_wr, &bad_send_wr))
		return; 
}

void write_inline(struct common *common, remote_var remote, uint32_t value, bool fence) {
    struct ibv_sge sge;
    struct ibv_send_wr send_wr = {};
    struct ibv_send_wr *bad_send_wr;

	common->local_buffer_for_inline_writes = value;

	/* Fill up SGE     */
	sge.addr                      = (uint64_t)(uintptr_t)&common->local_buffer_for_inline_writes;
	sge.length                    = sizeof (value);
	sge.lkey                      = common->inline_mr->lkey;
	
	/* Link to the send work request */
	send_wr.wr_id                 = 0;
	send_wr.opcode                = IBV_WR_RDMA_WRITE;  
	
	/* Set IBV_SEND_SIGNALED flag will cause an ibv event recv at sender when data transmit from memory to NIC */
	send_wr.send_flags = IBV_SEND_SIGNALED | IBV_SEND_INLINE;
	if (fence)
		send_wr.send_flags |= IBV_SEND_FENCE;

	send_wr.sg_list               = &sge;
	send_wr.num_sge               = 1;
	
	set_remote_var(common, &send_wr, remote);

	/* Post it */
	if (ibv_post_send(common->cm_id->qp, &send_wr, &bad_send_wr))
		return; 
}

void write_inline_w_imm(struct common *common, remote_var remote, uint32_t value, uint32_t immediate, bool fence) {
    struct ibv_sge sge;
    struct ibv_send_wr send_wr = {};
    struct ibv_send_wr *bad_send_wr;

	common->local_buffer_for_inline_writes = value;

	/* Fill up SGE     */
	sge.addr                      = (uint64_t)(uintptr_t)&common->local_buffer_for_inline_writes;
	sge.length                    = sizeof (value);
	sge.lkey                      = common->inline_mr->lkey;
	
	/* Link to the send work request */
	send_wr.wr_id                 = immediate;
	send_wr.opcode                = IBV_WR_RDMA_WRITE_WITH_IMM;
	send_wr.imm_data              = immediate;
	
	/* Set IBV_SEND_SIGNALED flag will cause an ibv event recv at sender when data transmit from memory to NIC */
	send_wr.send_flags = IBV_SEND_SIGNALED | IBV_SEND_INLINE;
	if (fence)
		send_wr.send_flags |= IBV_SEND_FENCE;

	send_wr.sg_list               = &sge;
	send_wr.num_sge               = 1;
	
	set_remote_var(common, &send_wr, remote);

	/* Post it */
	if (ibv_post_send(common->cm_id->qp, &send_wr, &bad_send_wr))
		return; 
}